    %%%%%%%%%%%%% Compliant tower calculation %%%%%%%%%%%%%%%%%%%%%

% Written by Ruben Martinez Ochagavia Spring 2018 (in Matlab R2016b)

%  This programme will calculate if a specific design for a compliant tower
%  fulfill the requirements for ULS and FLS. It will calculate from the
%  stiffness and mass matrix to the different loads acting on the structure
%  and the reactions in the structure for cyclic loading.
% 
%  The inputs needed for the calculation should be introduced in these functions:
%  --- CompliantTowerCalculation.m (current script)
%  --- CableT0.m (if a guy cable is use in the structure)
%  --- Checks.m
%  --- DynamicLoad.m
%  --- myode.m
%  --- StaticLoad.m (if a static load is introduced - not needed)(from previous calculations)
%  --- StiffnessGround.m (if the foundation is included)
% 
                        %%%%%%%
%%%  Description of input variables in CompliantTowerCalculation.m  %%% IS units

%  nel= number of elements
%  Beam= Type of beam element used for the stiffness matrix
%  E= Young's modulus
% Dbottom=Diameter (m) of the bottom of the tower
% tbottom= thickness (m) (it should be above 6mm -> NORSOK)
% Dtop=Diameter at the top of the tower
% ttop=thickness at the top
% Dtran=Diameter of the transition piece (from th seabed to the tower)
% ttran=thickness at the transition piece
% htp=height of the transition piece from the MSL to the end of the conical section of the tower
% density=efective density of the beam: kg/m3 (including 7850+paint+bolts+welds and flanges)
% tleng=total length of the beam
% rna =  mass of the rotor nacelle assembly kg
% inertiarna= inertia of the rna in kg*m2
% Wave= Yes(Y) or No(N)
% wdepth= Water depth from seabed
% Wind= Yes(Y) or No(N)
% Cable= Yes(Y) or No(N)
% MassTrap= Yes(Y) or No(N)
% hMTup=top height of the mass trap
% hMTdown=bottom height of the mass trap
% Dmt=Diameter of the mass trap
% Ground= Yes(Y) or No(N) If there is no ground included, there should be uncoupled springs,supported or campled system at the bottom
% Bottom=Supported (S) , campled (C), uncoupled springs(U) or Free ending (F). U just for no ground.
% For uncoupled springs-->kk11 and kk22 (transverse displacement stiffness and rotational stifness at the seabed, respectively)
% tvideo=time of the video showing the movement of the tower along time (s)
% 

clearvars
clc
disp('Wait. The code is still running.')

% Discretizing the Beam
nel=100;                 % number of elements
Beam='T'; %Type of beam chosen for the simulation: Timoshenko (T) or Bernoulli (B).

% Material properties
E=210*10^9;            % Youngs modulus: N/m2
Dbottom=4.5;                    % Diameter: m of the bottom of the tower
tbottom=0.030;             % thickness: m (it should be above 6mm -> NORSOK)
Dtop=3.5;
ttop=0.020; 
Dtran=4;                 %Diameter of the transition piece (from th seabed to the tower)
ttran=0.030;             %Thickness of the transition piece

htp=20; %height of the transition piece from the MSL to the end of the conical section of the tower

density = 8500;         %efective density of the beam: kg/m3 (including 7850+paint+bolts+welds and flanges)

tleng = 150;             % total length of the beam (140 m as example)

rna = 350000 ;          % mass of the rotor nacelle assembly kg
inertiarna=4034613;     % inertia of the rna in kg*m2

Wave='Y'; % Yes(Y) or No(N)
wdepth=50; % Water depth from seabed

Wind='Y'; % Yes(Y) or No(N)

Cable='Y'; % Yes(Y) or No(N)

MassTrap='N'; % Yes(Y) or No(N)
hMTup=40;
hMTdown=30;
Dmt=10; %Diameter of the mass trap (higher than the diameter of the tower at that points)

Ground='Y'; % Yes(Y) or No(N) If there is no ground included, there should be uncoupled springs,supported or campled system at the bottom
Bottom='C'; %Supported (S) , campled (C), uncoupled springs(U) or Free ending (F). U just for no ground.

tvideo=50;
%%%%%%%%%%%%%%%%%%%%%%%%

if Dbottom/tbottom>120 || Dtop/ttop>120
 disp('The relation between Diameter and Thickness is not fulfilled at the tower')  
end
if Dtran/ttran>120
  disp('The relation between Diameter and Thickness is not fulfilled at the substructure tower')   
end

%%%%%%%%%%%%%%%%%%%%%%%
if Bottom=='C'
bcdof =  [1 2]; %>campled at the bottom
nbcd=length(bcdof);
elseif Bottom=='S'
bcdof=[1];      %supported at the bottom
nbcd=length(bcdof);
elseif Bottom=='F' || Bottom=='U'
nbcd=0; %Free ending (controled just by the stiffness of the ground and other elements in the tower)
%For uncoupled springs%
kk11=24.5*10^6;%(N/m)
kk22=4130*10^6;%(N*m)
end

nnel=2;                 % number of nodes per element
ndof=2;                 % number of dofs per node
nnode=nel+1;   % total number of nodes in system
sdof=nnode*ndof;        % total system dofs 

leng = tleng/nel;       % uniform mesh (equal size of elements)
lengthvector = 0:leng:tleng;

if MassTrap=='Y'
[difference,NodeMassTrap1]=min(abs(lengthvector-(hMTdown)));  
[difference,NodeMassTrap2]=min(abs(lengthvector-(hMTup))); 
else
NodeMassTrap1=0;    
NodeMassTrap2=0; 
end

Dn=zeros(1,nnode);
tn=zeros(1,nnode);
[difference,NodeEndTower]=min(abs(lengthvector-(wdepth+htp)));   
for i=1:NodeEndTower-1       %Diameter and thickness along all the nodes above the seabed
 Dn(i)=Dtran;
 tn(i)=ttran;
end
for i=NodeEndTower:nnode                              
Dn(i)=Dbottom-((Dbottom-Dtop)*(i-NodeEndTower)/(nnode-NodeEndTower));
tn(i)=tbottom-((tbottom-ttop)*(i-NodeEndTower)/(nnode-NodeEndTower));
end

% Boundary Conditions       
kk=zeros(sdof,sdof);    % initialization of system stiffness matrix 
mm=zeros(sdof,sdof);    % initialization of system mass matrix 
% kkg=zeros(sdof,sdof);   % initialization of system geomtric stiffness matrix 
index=zeros(nel*ndof,1);  % initialization of index vector

kkhh=zeros(sdof,sdof);
for i=1:nel           % loop for the total number of elements
index=elementdof(i,nnel,ndof);  % vector containing system dofs associated with each element                           
[k,m,khh]=beam(E,Dn,tn,leng,i,density,Beam); % compute stiffness & mass matrix
kk=assemble(kk,k,index); % assemble element stiffness matrices into system matrix
mm=assemble(mm,m,index); % assemble element mass matrices into system matrix

kkhh=assemble(kkhh,khh,index);
end

%Defining the mass matrix for the water inside the tower (and in the mass trap if it is used)
[mw] = WaterMass(sdof,lengthvector,wdepth,nnel,ndof,Dn,tn,NodeMassTrap1,NodeMassTrap2,MassTrap,Dmt,leng);
%adding both mass matrices (steel and water)
mm=mm+mw;
%
%Adding the mass and inertia from the RNA
 mm(sdof-1,sdof-1) = mm(sdof-1,sdof-1)+ rna;      % Introducing the mass of the rotor nacelle assembly at the top
 mm(sdof,sdof) = mm(sdof,sdof)+ inertiarna;      % Introducing the polar inertia of the rna. 
 
%%%Adding stiffness from the cable in kk
%
if Cable=='Y'
[Th0,Tmax,DataCab,kcab] = CableT0(lengthvector);
kk(DataCab(5)*2-1,DataCab(5)*2-1)=kk(DataCab(5)*2-1,DataCab(5)*2-1)+kcab;
else
    Th0=0;
    Tmax=0;
    DataCab=0;
    kcab=0;
end

%%%Adding the ground
if Ground=='Y'
[kmp,mmp,nelmp,lengthvectormp,Lmp,Dnmp,tnmp,DataGr,kmg,LaboveG,LbelowG,khmp] = StiffnessGround(ndof,nnel,E,density,Beam);
    if Cable=='Y'
    DataCab(5)=DataCab(5)+nelmp; %New node for the guy cable
    end
    %Assemblying mass and stiffness matrixes from the foundation and the tower
nnodeF=nel+nelmp+1;
sdof=nnodeF*2;
kkf=zeros(sdof,sdof);
kkhf=zeros(sdof,sdof);
mmf=zeros(sdof,sdof);
indexgr=1:1:(nelmp+1)*2;
kkf=assemble(kkf,kmp,indexgr);
kkhf=assemble(kkhf,khmp,indexgr);
mmf=assemble(mmf,mmp,indexgr);
indexgr=(nelmp*2+1):1:sdof;
kkf=assemble(kkf,kk,indexgr);
kkhf=assemble(kkhf,kkhh,indexgr);
mmf=assemble(mmf,mm,indexgr);
lvhelp=zeros(1,size(lengthvectormp,2)-1);
for i=1:size(lengthvectormp,2)-1
lvhelp(i)=lengthvectormp(i)-Lmp;
end
lengthvectorF=[lvhelp,lengthvector];
DF=[Dnmp,Dn];
tF=[tnmp,tn];
if Bottom=='C' || Bottom=='S'
[kkf,mmf] = constraints(kkf,mmf,bcdof) ; 
[kkhf,mmf] = constraints(kkhf,mmf,bcdof) ;
end
elseif Ground=='N'
 %%%% Applying the Boundary conditions
 if Bottom=='C' || Bottom=='S'
[kk,mm] = constraints(kk,mm,bcdof) ;
[kkhh,mm] = constraints(kkhh,mm,bcdof) ;
 elseif Bottom=='U' %Uncoupled springs at the seabed
 kk(1,1)= kk(1,1)+ kk11; %(N/m) Displacement dof
 kk(2,2)= kk(2,2)+ kk22; %(N*m) Rotation dof
 end
kkf=kk;
kkhf=kkhh;
mmf=mm;
Lmp=0;
lengthvectorF=lengthvector;
DF=Dn;
tF=tn;
nnodeF=nnode;
nelmp=0;
DataGr=0;
kmg=0;
lengthvectormp=0;
LaboveG=0;
LbelowG=0;
end

%
%%% Natural frequencies 
[vecfreq, freq]=eig(kkf,mmf);   % solve the eigenvalue problem for Natural Frequencies
freq = diag(freq);
freq=sqrt(freq);  % UNITS :rad per sec

if nbcd>=1
    hp=1;
    for i=nbcd+1:4
  if freq(i)==1  %%%%% when the natural frequency is less than 1 rad/s% %Putting in order the frequencies taking into account the constraints (value of 1rad/s)
   vectorhelp=vecfreq(:,i);
   vecfreq(:,i)=vecfreq(:,hp);
   vecfreq(:,hp)=vectorhelp;
   freq(i)=freq(hp);
   freq(hp)=1;
   hp=hp+1;
  end
    end
    if freq(nbcd+1)>freq(nbcd+2)
   vectorhelp=vecfreq(:,nbcd+1);
   vecfreq(:,nbcd+1)=vecfreq(:,nbcd+2);
   vecfreq(:,nbcd+2)=vectorhelp; 
   freqhelp=freq(nbcd+1);
   freq(nbcd+1)=freq(nbcd+2);
   freq(nbcd+2)=freqhelp;   
    end
end
freqHz = freq/(2*pi); % UNITS : Hertz
%
%
%Plotting the mode shapes
h = figure ;
set(h,'name','Mode Shapes of Beam in Hz','numbertitle','off')
PlotModeShapes(vecfreq,freqHz,lengthvectorF,nbcd) %nbcd for clamped and 0 for ground
%
%%%%%%%%

% [xMax]=StaticLoad(sdof,lengthvectorF,kkf); %Just for checking that
% calculations were correct (Not needed anymore)

[Vnode,Mnode,Vdc,Mdc,Vcampled,Mcampled,uf,MaxAc,tf]=DynamicLoad(kkhf,mmf,sdof,nnodeF,freq,vecfreq,kkf,Bottom,Wind,Wave,...
    lengthvectorF,rna,wdepth,DF,leng,nelmp,nbcd,DataGr,kmg,lengthvectormp,LaboveG,LbelowG,Ground,Th0,Tmax,DataCab,kcab,Cable);

xDynMax=max(abs(uf(:,sdof-1))); %at the top

MaxTilt=max(abs(uf(:,sdof)))*360/(2*pi);%in deg

%Checking the structure according to the guidelines
[Dmax,jDmax,Dtotal]=Checks(wdepth,Cable,DataCab,Tmax,Ground,DF,tF,uf,nelmp,Lmp,nnodeF,E,nnode,leng,Mnode,...
    Vnode,Mdc,Vdc,lengthvectorF,rna,density,Bottom,Vcampled,Mcampled,tf);


%Video of displacements
disp(' ')
prompt = 'Do you want to see a video with the displacements along the tower? (Y) or (N) ';
str = input(prompt,'s');
if isempty(str)
    str = 'Y';
end
if str=='Y' || str=='y'
ufdisp=uf(:,1:2:end);
xw=-xDynMax*3:0.1:xDynMax*3;
yw=cos(xw)+wdepth; %Example to show the MSL
ys=0*xw;

tsimulation=10*tvideo;
set(0, 'DefaultFigureVisible', 'off');
fig=figure;
for i= 1:tsimulation %select the time it is running: size(ufdisp,1) is all the simulation
    set(0, 'currentfigure', fig);
    plot(xw,yw,'k')
    hold on
    plot(xw,ys,'k')
    plot(ufdisp(i,:),lengthvectorF,'*')
    axis([-xDynMax*2 ,xDynMax*2,min(lengthvectorF),tleng])
    hold off
    Mov(i)=getframe;
    
end
set(0, 'DefaultFigureVisible', 'on');
fig=figure;
axis([-xDynMax*2 ,xDynMax*2,min(lengthvectorF),tleng])
movie(Mov,2,10)
end

clear bcdof DataCab DataGr difference Dn Dnmp hMTdown hMTup hp i index indexgr k kk kmp LaboveG LbelowG...
    leng lengthvector lengthvectormp lvhelp m Mcampled Mdc mm mmp mw nbcd ndof nel nelmp nnel nnode NodeEndTower...
    NodeMassTrap1 NodeMassTrap2 prompt str Th0 tn tnmp Vcampled Vdc vectorhelp xw ys yw tsimulation
 
disp('Program finished')