function [kmp,mmp,nelmp,lengthvectormp,Lmp,Dnmp,tnmp,DataGr,kmg,LaboveG,LbelowG,khmp] = StiffnessGround( ndof,nnel,E,density,Beam)

%%%  Function Description:
%This function is used to get characteristic information about the
%soil-structure interaction after introducing the data of the ground and it
%also gets the stiffness matrix and mass matrix for the monopile into the
%seabed

%%%  Input Variables Description: %%%
% Lmp=length of the monopile (from the seabed) (m)
% Dmp=diameter of the monopile (m)
% tmp=thickness of the monopile (m)
% nelmp=Number of elements in the monopile
% Labove=Length of the upper surface. Equal to Lmp if it is just one type of soil
% LaboveG and LbelowG(Type of soil)--> Clay  or  Sand
% SuwC=(kN/m3)Saturated unit weight of Clay
% SuwS=(kN/m3)Saturated unit weight of Sand
% cu=Undrained shear strength for undisturbed clay soil samples (kPa)
% ec=strain which occurs at one-half the maximum stress on laboratory
% ...unconsolidated undrained compression tests of undisturbed soil samples in %
% HR=Depth below soil surface to bottom of reduced resistance zone (m). Minimum recommended value selected. 
% J=dimensionless empirical constant with values ranging from 0.25 to 0.5 having been determined by field testing
% C1,C2 and C3= 3 Coefficients depending on the angle of internal friction of sand (38)
% ksand=(kN/m3) Initial modulus of subgrade reaction depending on the angle of internal friction 
% A= factor for cyclic loading (0.9)
%                 

%%%%%%%%%%%
Lmp=50; % length of the monopile (from the seabed) (m)
Dmp=4; % diameter of the monopile (m)
tmp=0.034; %thickness of the monopile (m)
nelmp=50; % Number of elements in the monopile

Labove=50; %Length of the upper surface. Equal to Lmp if it is just one type of soil
LaboveG='Clay'; %(Type of soil) %Calculations for soft clay or sand--> Clay  or  Sand
Lbelow=Lmp-Labove;
LbelowG='Sand';

SuwC=19; % (kN/m3)Saturated unit weight of Clay
SuwS=19; % (kN/m3)Saturated unit weight of Sand

SuwW=9.81; % (kN/m3)Saturated unit weight of water
EfswC=SuwC-SuwW; %(kN/m3) Effective soil weight of Clay
EfswS=SuwS-SuwW; %(kN/m3) Effective soil weight of Sand 
satdensityC=SuwC*1000/9.81; %Saturated density of Clay
satdensityS=SuwS*1000/9.81; %Saturated densiry of Sand

cu=20; % Undrained shear strength for undisturbed clay soil samples (kPa)
ec=0.01; % strain which occurs at one-half the maximum stress on laboratory
         %unconsolidated undrained compression tests of undisturbed soil samples in %
yc=2.5*ec*Dmp; %(m)

HR=2.5*Dmp; %Depth below soil surface to bottom of reduced resistance zone (m). Minimum recommended value. It can be changed. 
J=0.5; %dimensionless empirical constant with values ranging from 0.25 to 0.5 having been determined by field testing

% 3 Coefficients depending on the angle of internal friction of sand (38)
C1=3.8;   %3.2 for 38
C2=4;
C3=76;
ksand=120*271.447138; % (kN/m3) Initial modulus of subgrade reaction depending on the angle of internal friction 
A=0.9; %for cyclic loading

if Dmp/tmp>120
  disp('The relation between Diameter and Thickness is not fulfilled at the foundation')   
end

nnodemp=nelmp+1;  % total number of nodes in system of the monopile
sdofmp=nnodemp*ndof;        % total system dofs of the monopile

lengmp = Lmp/nelmp;% uniform mesh (equal size of elements)
lengthvectormp = 0:lengmp:Lmp; %0 at the lowest level and Lmp at the ground level

Dnmp=zeros(1,nelmp);
tnmp=zeros(1,nelmp);
for i=1:nelmp       %Diameter and thickness along all the nodes of the monopile but the last one (will be the diameter of the tower)
 Dnmp(i)=Dmp;
 tnmp(i)=tmp;
end

kmp=zeros(sdofmp,sdofmp);    % initialization of system stiffness matrix 
khmp=zeros(sdofmp,sdofmp);
mmp=zeros(sdofmp,sdofmp);    % initialization of system mass matrix
mg=zeros(sdofmp,sdofmp);
indexmp=zeros(nelmp*ndof,1);  % initialization of index vector

for i=1:nelmp           % loop for the total number of elements
indexmp=elementdof(i,nnel,ndof);  % extract system dofs associated with element                        
[k,m,khg]=beammp(E,Dmp,tmp,lengmp,density,Beam); % compute stiffness & mass matrix
kmp=assemble(kmp,k,indexmp); % assemble element stiffness matrices into system matrix
mmp=assemble(mmp,m,indexmp); % assemble element mass matrices into system matrix
khmp=assemble(khmp,khg,indexmp);
end

[difference,NodeLayer]=min(abs(lengthvectormp-(Lbelow))); 
kmg=zeros(nnodemp-1,1);

for i=1:nnodemp-1 %%%%%introducing the stiffness of the ground in each node
  if i>=NodeLayer
      Ground=LaboveG;
  else
      Ground=LbelowG;
  end
  H=Lmp-lengthvectormp(i);%Depth below soil surface (m)
  if Ground=='Clay'
    if H<HR
    pu=3*cu+EfswC/10^3*H*1000+J*cu*H*1000/(Dmp*1000); % ultimate resistance (kPa)
    p=pu*1000*[0;0.23;0.33;0.5;0.72;0.72*H/HR;0.72*H/HR]; %(N/m2)
    yg=yc*[0;0.1;0.3;1;3;15;1000];
    else
    pu=9*cu; %(kPa)
    p=pu*1000*[0;0.23;0.33;0.5;0.72;0.72]; %(N/m2)
    yg=yc*[0;0.1;0.3;1;3;1000];
    end
        if i==1
         p=p*Dmp*lengmp/2; %Force in N
        else
         p=p*Dmp*lengmp;    %Force in N
        end
    %stiffness for the displacement at the beginning
    kmg(i)=p(2)/yg(2);
    kmp(i*2-1,i*2-1)=kmp(i*2-1,i*2-1)+kmg(i);
    khmp(i*2-1,i*2-1)=khmp(i*2-1,i*2-1)+kmg(i);
  end

if Ground=='Sand'
 pus=(C1*H+C2*Dmp)*EfswS*H;
 pud=C3*Dmp*EfswS*H;
 pu=min(pus,pud); %kN/m
 yg=0:0.01:2;
 yg=[yg,100];
 p=1000*A*pu*tanh(ksand*H/A/pu*yg); %(N/m)
  if i==1
     p=p*lengmp/2; %Force in N
  else
     p=p*lengmp;   %Force in N
  end
   kmg(i)=p(2)/yg(2); %stiffness for a displacement of 0.01m (at the beginning)
   kmp(i*2-1,i*2-1)=kmp(i*2-1,i*2-1)+kmg(i);
   khmp(i*2-1,i*2-1)=khmp(i*2-1,i*2-1)+kmg(i);
end
%       fig=figure;
%       plot(yg,p)
%       axis([0 yg(5) 0 p(5)]) %%% p-y curve
end
for i=1:nelmp %introducing the mass of ground inside the monopile in each element
     if i>=NodeLayer
      Ground=LaboveG;
     else
      Ground=LbelowG;
     end 
 if Ground=='Clay'
indexmp=elementdof(i,nnel,ndof);  % extract system dofs associated with element                        
A=pi/4*(Dmp-2*tmp)^2;         %Area of the element
mass=satdensityC*A;
    mm1=mass*lengmp/420;
    m=mm1*[156      22*lengmp   54       -13*lengmp;...
          22*lengmp  4*lengmp^2  13*lengmp  -3*lengmp^2;...
          54       13*lengmp   156      -22*lengmp;...
         -13*lengmp -3*lengmp^2 -22*lengmp   4*lengmp^2];
mg=assemble(mg,m,indexmp); % assemble element mass matrices into system matrix for the water  
end        
 if Ground=='Sand'
 indexmp=elementdof(i,nnel,ndof);  % extract system dofs associated with element                        
A=pi/4*(Dmp-2*tmp)^2;         %Area of the element
mass=satdensityS*A;
    mm1=mass*lengmp/420;
    m=mm1*[156      22*lengmp   54       -13*lengmp;...
          22*lengmp  4*lengmp^2  13*lengmp  -3*lengmp^2;...
          54       13*lengmp   156      -22*lengmp;...
         -13*lengmp -3*lengmp^2 -22*lengmp   4*lengmp^2];
mg=assemble(mg,m,indexmp); % assemble element mass matrices into system matrix for the water 
end   
end
mmp=mmp+mg;
%Keeping all the data in just one vector to move it to other functions easily
DataGr=[Dmp Lmp Labove Lbelow SuwC SuwS cu ec J C1 C2 C3 ksand A];
end

